@@ -1,4 +1,4 @@
-use std::{fmt, mem};
+use std::{fmt, mem, sync::Arc};
 
 use futures_intrusive::sync::ManualResetEvent;
 use metrics::{decrement_gauge, increment_counter, increment_gauge};
@@ -12,24 +12,23 @@ use crate::{
     errors::{SendError, TryRecvError, TrySendError},
     group::TerminationPolicy,
     mailbox::Mailbox,
-    messages::Terminate,
+    messages::{ActorStatusReport, Terminate},
     request_table::RequestTable,
+    subscription::SubscriptionManager,
 };
 
 use crate::{self as elfo};
 use elfo_macros::msg_raw as msg;
 
-pub(crate) struct Actor {
-    termination_policy: TerminationPolicy,
-    mailbox: Mailbox,
-    request_table: RequestTable,
-    control: RwLock<ControlBlock>,
-    finished: ManualResetEvent,
+// === ActorMeta ===
+
+#[derive(Debug, Hash, PartialEq, Serialize, Deserialize)]
+pub struct ActorMeta {
+    pub group: String,
+    pub key: String,
 }
 
-struct ControlBlock {
-    status: ActorStatus,
-}
+// === ActorStatus ===
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct ActorStatus {
@@ -37,8 +36,64 @@ pub struct ActorStatus {
     details: Option<String>,
 }
 
-#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
-enum ActorStatusKind {
+impl ActorStatus {
+    pub const ALARMING: ActorStatus = ActorStatus::new(ActorStatusKind::Alarming);
+    pub(crate) const FAILED: ActorStatus = ActorStatus::new(ActorStatusKind::Failed);
+    pub const INITIALIZING: ActorStatus = ActorStatus::new(ActorStatusKind::Initializing);
+    pub const NORMAL: ActorStatus = ActorStatus::new(ActorStatusKind::Normal);
+    pub(crate) const TERMINATED: ActorStatus = ActorStatus::new(ActorStatusKind::Terminated);
+    pub const TERMINATING: ActorStatus = ActorStatus::new(ActorStatusKind::Terminating);
+
+    const fn new(kind: ActorStatusKind) -> Self {
+        Self {
+            kind,
+            details: None,
+        }
+    }
+
+    /// Creates a new status with the same kind and provided details.
+    pub fn with_details(&self, details: impl fmt::Display) -> Self {
+        ActorStatus {
+            kind: self.kind,
+            details: Some(format!("{}", details)),
+        }
+    }
+
+    /// Returns the corresponding [`ActorStatusKind`] for this status.
+    pub fn kind(&self) -> ActorStatusKind {
+        self.kind
+    }
+
+    /// Returns details for this status, if provided.
+    pub fn details(&self) -> Option<&str> {
+        self.details.as_deref()
+    }
+
+    pub(crate) fn is_failed(&self) -> bool {
+        self.kind == ActorStatusKind::Failed
+    }
+
+    fn is_finished(&self) -> bool {
+        use ActorStatusKind::*;
+        matches!(self.kind, Failed | Terminated)
+    }
+}
+
+impl fmt::Display for ActorStatus {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match &self.details {
+            Some(details) => write!(f, "{:?}: {}", self.kind, details),
+            None => write!(f, "{:?}", self.kind),
+        }
+    }
+}
+
+// === ActorStatusKind ===
+
+/// A list specifying statuses of actors. It's used with the [`ActorStatus`].
+#[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
+#[non_exhaustive]
+pub enum ActorStatusKind {
     Normal,
     Initializing,
     Terminating,
@@ -60,50 +115,31 @@ impl ActorStatusKind {
     }
 }
 
-impl fmt::Display for ActorStatus {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match &self.details {
-            Some(details) => write!(f, "{:?}: {}", self.kind, details),
-            None => write!(f, "{:?}", self.kind),
-        }
-    }
+// === Actor ===
+
+pub(crate) struct Actor {
+    meta: Arc<ActorMeta>,
+    termination_policy: TerminationPolicy,
+    mailbox: Mailbox,
+    request_table: RequestTable,
+    control: RwLock<ControlBlock>,
+    finished: ManualResetEvent, // TODO: remove in favor of `status_subscription`?
+    status_subscription: Arc<SubscriptionManager>,
 }
 
-impl ActorStatus {
-    pub const ALARMING: ActorStatus = ActorStatus::new(ActorStatusKind::Alarming);
-    pub(crate) const FAILED: ActorStatus = ActorStatus::new(ActorStatusKind::Failed);
-    pub const INITIALIZING: ActorStatus = ActorStatus::new(ActorStatusKind::Initializing);
-    pub const NORMAL: ActorStatus = ActorStatus::new(ActorStatusKind::Normal);
-    pub(crate) const TERMINATED: ActorStatus = ActorStatus::new(ActorStatusKind::Terminated);
-    pub const TERMINATING: ActorStatus = ActorStatus::new(ActorStatusKind::Terminating);
-
-    const fn new(kind: ActorStatusKind) -> Self {
-        Self {
-            kind,
-            details: None,
-        }
-    }
-
-    pub fn with_details(&self, details: impl fmt::Display) -> Self {
-        ActorStatus {
-            kind: self.kind.clone(),
-            details: Some(format!("{}", details)),
-        }
-    }
-
-    pub(crate) fn is_failed(&self) -> bool {
-        self.kind == ActorStatusKind::Failed
-    }
-
-    fn is_finished(&self) -> bool {
-        use ActorStatusKind::*;
-        matches!(self.kind, Failed | Terminated)
-    }
+struct ControlBlock {
+    status: ActorStatus,
 }
 
 impl Actor {
-    pub(crate) fn new(addr: Addr, termination_policy: TerminationPolicy) -> Self {
+    pub(crate) fn new(
+        meta: Arc<ActorMeta>,
+        addr: Addr,
+        termination_policy: TerminationPolicy,
+        status_subscription: Arc<SubscriptionManager>,
+    ) -> Self {
         Actor {
+            meta,
             termination_policy,
             mailbox: Mailbox::new(),
             request_table: RequestTable::new(addr),
@@ -111,6 +147,7 @@ impl Actor {
                 status: ActorStatus::INITIALIZING,
             }),
             finished: ManualResetEvent::new(false),
+            status_subscription,
         }
     }
 
@@ -119,6 +156,8 @@ impl Actor {
             "status" => ActorStatusKind::Initializing.as_str());
         increment_counter!("elfo_actor_status_changes_total",
             "status" => ActorStatusKind::Initializing.as_str());
+
+        self.send_status_to_subscribers(&self.control.read());
     }
 
     pub(crate) fn try_send(&self, envelope: Envelope) -> Result<(), TrySendError<Envelope>> {
@@ -169,12 +208,14 @@ impl Actor {
     pub(crate) fn set_status(&self, status: ActorStatus) {
         let mut control = self.control.write();
         let prev_status = mem::replace(&mut control.status, status.clone());
-        drop(control);
 
         if status == prev_status {
             return;
         }
 
+        self.send_status_to_subscribers(&control);
+        drop(control);
+
         if status.is_finished() {
             self.mailbox.close();
             self.finished.set();
@@ -209,6 +250,7 @@ impl Actor {
         }
 
         // TODO: use `sdnotify` to provide a detailed status to systemd.
+        //       or use another actor to listen all statuses for this.
     }
 
     pub(crate) fn close(&self) -> bool {
@@ -222,18 +264,47 @@ impl Actor {
         )
     }
 
+    pub(crate) fn is_terminating(&self) -> bool {
+        matches!(
+            self.control.read().status.kind,
+            ActorStatusKind::Terminating
+        )
+    }
+
     pub(crate) async fn finished(&self) {
         self.finished.wait().await
     }
+
+    /// Accesses the actor's status under lock to avoid race conditions.
+    pub(crate) fn with_status<R>(&self, f: impl FnOnce(ActorStatusReport) -> R) -> R {
+        let control = self.control.read();
+        f(ActorStatusReport {
+            meta: self.meta.clone(),
+            status: control.status.clone(),
+        })
+    }
+
+    fn send_status_to_subscribers(&self, control: &ControlBlock) {
+        self.status_subscription.send(ActorStatusReport {
+            meta: self.meta.clone(),
+            status: control.status.clone(),
+        });
+    }
 }
 
 #[cfg(test)]
+#[cfg(feature = "FIXME")]
 mod tests {
     use super::*;
 
     #[tokio::test]
     async fn finished() {
-        let actor = Actor::new(Addr::NULL, TerminationPolicy::default());
+        let meta = Arc::new(ActorMeta {
+            group: "foo".into(),
+            key: "bar".into(),
+        });
+
+        let actor = Actor::new(meta, Addr::NULL, TerminationPolicy::default());
         let fut = actor.finished();
         actor.set_status(ActorStatus::TERMINATED);
         fut.await;
