@@ -3,8 +3,8 @@ use std::{cell::Cell, future::Future, sync::Arc};
 use elfo_utils::RateLimiter;
 
 use crate::{
+    actor::ActorMeta,
     addr::Addr,
-    object::ObjectMeta,
     permissions::{AtomicPermissions, Permissions},
     trace_id::{self, TraceId},
 };
@@ -15,9 +15,9 @@ tokio::task_local! {
 
 #[derive(Clone)]
 pub struct Scope {
-    addr: Addr,
+    actor: Addr,
     group: Addr,
-    meta: Arc<ObjectMeta>,
+    meta: Arc<ActorMeta>,
     trace_id: Cell<TraceId>,
 
     // Per group.
@@ -32,15 +32,15 @@ impl Scope {
     /// Private API for now.
     #[doc(hidden)]
     pub fn new(
-        addr: Addr,
+        actor: Addr,
         group: Addr,
-        meta: Arc<ObjectMeta>,
+        meta: Arc<ActorMeta>,
         perm: Arc<AtomicPermissions>,
         logging_limiter: Arc<RateLimiter>,
     ) -> Self {
         Self::with_trace_id(
             trace_id::generate(),
-            addr,
+            actor,
             group,
             meta,
             perm,
@@ -52,14 +52,14 @@ impl Scope {
     #[doc(hidden)]
     pub fn with_trace_id(
         trace_id: TraceId,
-        addr: Addr,
+        actor: Addr,
         group: Addr,
-        meta: Arc<ObjectMeta>,
+        meta: Arc<ActorMeta>,
         permissions: Arc<AtomicPermissions>,
         logging_limiter: Arc<RateLimiter>,
     ) -> Self {
         Self {
-            addr,
+            actor,
             group,
             meta,
             trace_id: Cell::new(trace_id),
@@ -69,8 +69,14 @@ impl Scope {
     }
 
     #[inline]
+    #[deprecated(note = "use `actor()` instead")]
     pub fn addr(&self) -> Addr {
-        self.addr
+        self.actor
+    }
+
+    #[inline]
+    pub fn actor(&self) -> Addr {
+        self.actor
     }
 
     #[inline]
@@ -80,7 +86,7 @@ impl Scope {
 
     /// Returns the current object's meta.
     #[inline]
-    pub fn meta(&self) -> &Arc<ObjectMeta> {
+    pub fn meta(&self) -> &Arc<ActorMeta> {
         &self.meta
     }
 
@@ -180,12 +186,12 @@ pub fn set_trace_id(trace_id: TraceId) {
 /// # Panics
 /// This function will panic if called ouside the actor system.
 #[inline]
-pub fn meta() -> Arc<ObjectMeta> {
+pub fn meta() -> Arc<ActorMeta> {
     with(|scope| scope.meta().clone())
 }
 
 /// Returns the current object's meta if inside the actor system.
 #[inline]
-pub fn try_meta() -> Option<Arc<ObjectMeta>> {
+pub fn try_meta() -> Option<Arc<ActorMeta>> {
     try_with(|scope| scope.meta().clone())
 }
