@@ -10,17 +10,18 @@ use tracing::{error, info, warn, Level};
 use elfo_macros::message;
 
 use crate::{
-    actor::Actor,
+    actor::{Actor, ActorMeta},
     addr::Addr,
     context::Context,
     demux::Demux,
     errors::{RequestError, StartError},
     message,
     messages::{Ping, Terminate, UpdateConfig},
-    object::{Object, ObjectMeta},
+    object::Object,
     permissions::{AtomicPermissions, Permissions},
     scope::Scope,
     signal::{Signal, SignalKind},
+    subscription::SubscriptionManager,
     topology::Topology,
 };
 
@@ -102,16 +103,25 @@ pub async fn do_start<F: Future>(
 
     let entry = topology.book.vacant_entry();
     let addr = entry.addr();
-    entry.insert(Object::new(addr, Actor::new(addr, Default::default())));
-
     let dumper = topology.dumper.for_group(false); // TODO: should we dump the starter?
+    let ctx = Context::new(topology.book.clone(), dumper, Demux::default()).with_addr(addr);
 
-    let meta = ObjectMeta {
+    let meta = Arc::new(ActorMeta {
         group: "init".into(),
-        key: Some("_".into()), // Just like `Singleton`.
-    };
+        key: "_".into(), // Just like `Singleton`.
+    });
+
+    // XXX
+    entry.insert(Object::new(
+        addr,
+        Actor::new(
+            meta.clone(),
+            addr,
+            Default::default(),
+            Arc::new(SubscriptionManager::new(ctx.clone())),
+        ),
+    ));
 
-    // XXX: `addr` is used for both a specific actor and a whole group for now.
     let perm = Arc::new(AtomicPermissions::default());
     perm.store({
         let mut perm = Permissions::default();
@@ -119,9 +129,8 @@ pub async fn do_start<F: Future>(
         perm
     });
 
-    let scope = Scope::new(addr, addr, Arc::new(meta), perm, Default::default());
+    let scope = Scope::new(addr, Addr::NULL, meta, perm, Default::default());
     let f = async move {
-        let ctx = Context::new(topology.book.clone(), dumper, Demux::default()).with_addr(addr);
         start_entrypoints(&ctx, &topology).await?;
         wait_entrypoints(&ctx, &topology).await?;
         Ok(f(ctx, topology).await)
